const path = require('path')
const webpack = require('webpack')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const HtmlWebpackTagsPlugin = require('html-webpack-tags-plugin')

const ProgressBarPlugin = require('progress-bar-webpack-plugin')
const VueLoaderOptionsPlugin = require('vue-loader-options-plugin')

const dllManifestDir = path.resolve(__dirname, './build/manifest/prod')

const resolve = _path => path.resolve(__dirname, `${_path}`)

const webpackConfig = {
  mode: 'development',
  devtool: 'eval-cheap-module-source-map',
  entry: {
    main: [
      'webpack-hot-middleware/client?noInfo=false&reload=true',
      resolve('./src/main')
    ]
  },
  output: {
    filename: 'js/[name].bundle.js',
    chunkFilename: 'js/chunks/[id].[chunkhash:8].js',
    publicPath: '/'
  },
  resolve: {
    extensions: ['.js', '.jsx', '.vue', '.json'],
    modules: [path.resolve(__dirname, 'node_modules'), 'node_modules']
  },
  resolveLoader: {
    modules: [path.resolve(__dirname, 'node_modules'), 'node_modules']
  },
  externals: {
    'dmp-chart-sdk': 'DmpChartSDK'
  },
  watchOptions: {
    aggregateTimeout: 300,
    poll: 1000
  },
  module: {
    rules: [
      {
        test: /\.vue$/,
        loader: 'vue-loader'
      },
      {
        test: /\.(js|jsx)$/,
        loader: 'babel-loader?cacheDirectory=true',
        include: [resolve('src'), resolve('chartlibs')],
        exclude: [resolve('node_modules'), resolve('src/components/dmpsimulator')],
        options: {
          extends: path.resolve(__dirname, './.babelrc')
        }
      },
      {
        test: /\.(png|jpe?g|gif|ico)(\?.*)?$/,
        loader: 'url-loader',
        query: {
          limit: 81920,
          name: 'images/[name].[hash:7].[ext]'
        }
      },
      {
        test: /\.(woff2?|eot|ttf|otf|svg)(\?.*)?$/,
        loader: 'url-loader',
        query: {
          limit: 8192,
          name: 'fonts/[name].[hash:7].[ext]'
        }
      },
      {
        test: /\.css$/,
        use: [
          'style-loader',
          'css-loader'
        ]
      },
      {
        test: /^(.*\.module).*\.less/,
        use: [
          'style-loader',
          {
            loader: 'css-loader',
            options: {
              importLoaders: 1,
              modules: true,
              localIdentName: '[local]--[hash:base64:5]',
            }
          },
          {
            loader: 'less-loader',
            options: {
              importLoaders: 2,
              modules: true,
              localIdentName: '[local]--[hash:base64:5]',
            },
          },
        ]
      },
      {
        test: /^(?!.*\.module).*\.less/,
        use: [
          'style-loader',
          'css-loader',
          'less-loader'
        ]
      },
    ]
  },
  plugins: [
    new ProgressBarPlugin(),
    new webpack.HotModuleReplacementPlugin(),
    new webpack.NoEmitOnErrorsPlugin(),
    new VueLoaderOptionsPlugin({
      babel: {
        extends: path.resolve(__dirname, './.babelrc')
      }
    }),
    new webpack.DllReferencePlugin({
      context: dllManifestDir,
      manifest: require(path.join(dllManifestDir, 'vendor-manifest.json'))
    }),
    new HtmlWebpackPlugin({
      filename: 'index.html',
      template: path.resolve(__dirname, './index.html'),
      inject: true
    }),
    new HtmlWebpackTagsPlugin({
      tags: [
        '/chartsdk/prod/dmp-chart-sdk.js',
        'https://dmp-open.mypaas.com.cn/dmp-static/comlib/f2.20fa8c4d.js',
      ],
      append: false,
      hash: true,
      publicPath: '',
    })
  ]
}

module.exports = webpackConfig
